/* * This file is part of NFC Middleware *
 *
 * Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Hannu Mallat <hannu.mallat@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */

/*!
 * @file    NDEFPusher.h
 * @brief   Device to device NDEF message push API
 *
 * @author  Hannu Mallat
 * @date    14/07/2011
 *
 * @copyright Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 *
 * @license All rights reserved.
 */

#ifndef NDEF_PUSHER_H
#define NDEF_PUSHER_H

#include <QObject>

#include <QNdefMessage>

class NDEFPusherPrivate;
class NDEFPushRequest;

/*!
  \class NDEFPusher
  
  \brief The NDEFPusher class provides means to push NDEF messages
  between devices.

  NDEF messages can be pushed from device to device in various ways,
  including tag emulation and LLCP-based protocols such as SNEP
  (Simple NDEF Exchange Protocol). This class provides a mechanism for
  an application to push messages to other devices without having to
  bother with protocol-specific details. 
 
  An application should first create a NDEFPusher object and then
  request push service by requestPushService(). Once the service is
  available, the application can place NDEF messages into an outgoing
  queue using the push() method. Each message placed in the outgoing
  queue will be assigned an ID that can be used to cancel the push,
  and it is also used when NDEFPusher notifies the application of push
  transmission completion.

 */
class NDEFPusher : public QObject
{
        Q_OBJECT;
	Q_ENUMS(RequestStatus);
	Q_DECLARE_PRIVATE(NDEFPusher);

public:
	/*! \brief Push request status. */
	enum RequestStatus {
		REQUEST_OK, /*!< Request succeeded */
		REQUEST_FAILED, /*!< Request failed */
		REQUEST_PEER_LOST, /*!< Request failed due to lost connection */
		REQUEST_PEER_REJECTED, /*!< Request failed due to rejection by receiver */
		REQUEST_PEER_LIMITS_EXCEEDED /*!< Request failed due to receiver resource limits */
	};

	/*!

	  Constructs a new NDEF pusher with \a parent. 

	*/
	Q_INVOKABLE NDEFPusher(QObject *parent = 0);

	/*!

	  Destroys the NDEF pusher.

	*/
	virtual ~NDEFPusher(void);

	/*!  

	  Request push service. When the request is completed, one of
	  the signals pushServiceAvailable() and
	  pushServiceUnavailable() will be emitted.

	  Also, if the push service becomes spontaneously available or
	  unavailable the signals will be emitted.

	*/
	Q_INVOKABLE bool requestPushService(void);

	/*!

	  Check whether the push service is currently available. 

	*/
	Q_INVOKABLE bool isPushServiceAvailable(void) const;

	/*!  

	  Place an NDEF message to the outgoing queue. Push service
	  must be available for the call to succeed. Message delivery
	  will be attempted when a peer device that can receive the
	  message is detected.

	  Returns true if the message is succesfully placed in the
	  queue, false otherwise. If true is returned, \a id will
	  contain a push ID that is used to identify the message.

	  Once message transfer starts, pushProgress() signals will be
	  emitted, informing the application of transfer
	  progress. When message transfer is completed, succesfully or
	  not, pushComplete() signal will be emitted.

	*/
	Q_INVOKABLE bool push(const QtMobility::QNdefMessage &message,
			      unsigned int &id);

	/*!

	  Remove a message from the outgoing queue. Note that this can
	  be done only if the message transfer has not yet
	  started. The message in the queue is identified by \a id.

	  Returns true if the message was removed.
	
	*/
	Q_INVOKABLE bool cancelPush(unsigned int id);

Q_SIGNALS:
	/*!
	  Emitted when push service becomes available. 
	*/
	void pushServiceAvailable(void);

	/*!
	  Emitted when push service becomes unavailable. 
	*/
	void pushServiceUnavailable(void);

	/*!
	  Emitted when a push request completes.
	*/
	void pushComplete(unsigned int id, 
			  enum NDEFPusher::RequestStatus status);

	/*!
	  Emitted when some data is transferred for a push request.
	*/
	void pushProgress(unsigned int id,
			  quint64 total,
			  quint64 done);

private:
	NDEFPusherPrivate *d_ptr;

};

#endif /* NDEF_PUSHER_H */
